@@ -16,8 +16,9 @@ class Argument:
     optional = False # May be excluded from input.
     save = False # Save value between executions.
     value = False # Pass by value, not by reference.
-    pointer = False # Pass by pointer, not by reference.
     parameter = False # Whether this argument is a parameter (set at compile time)
+    target = False # Whether this argument is a valid pointer target (set at compile time).
+    pointer = False # Whether this argument is a pointer (set at compile time).
     type_len = False # Whether this argument has the LEN parameter (in a derived TYPE)
     type_kind = False # Whether this argument has the KIND parameter (in a derived TYPE)
     dimension = None # If array, this list describes the shape and size.
@@ -48,7 +49,9 @@ class Argument:
     #          size checking will have an exact 1-1 mapping of arguments.
     def function_safe_name(self):
         name = self.name
-        if (self.parent is not None) and (self.parent.name == self.name):
+        if (self.allocatable):
+            name += "_LOCAL"
+        elif (self.parent is not None) and (self.parent.name == self.name):
             name += "_RESULT"
         return name
 
@@ -60,10 +63,10 @@ class Argument:
     #          Fortran call will have an exact 1-1 mapping of arguments.
     def fort_call_name(self):
         name = self.name
-        if (self.parent is not None) and (self.parent.name == self.name):
-            name += "_RESULT"
-        elif (self.allocatable):
+        if (self.allocatable):
             name += "_LOCAL"
+        elif (self.parent is not None) and (self.parent.name == self.name):
+            name += "_RESULT"
         return name
 
     # Names of arguments that will be given to the Fortran wrapper.
@@ -87,7 +90,7 @@ class Argument:
     def fort_declare(self):
         lines = []
         if (self.optional):
-            lines.append(f"LOGICAL, INTENT(IN) :: {self.name}_PRESENT")
+            lines.append(f"LOGICAL(KIND=C_BOOL), INTENT(IN) :: {self.name}_PRESENT")
         # If this is an allocatable, the sizes have to be given as output.
         size_type = "IN" if not self.allocatable else "OUT"
         # Add extra arguments for the dimension sizes.
@@ -139,13 +142,13 @@ class Argument:
         if (is_output and self.allocatable):
             if (self.dimension is None): raise NotImplementedError
             for i in range(1,len(self.dimension)+1):
-                if present: size = f"SIZE({self.function_safe_name()}_LOCAL,{i})"
+                if present: size = f"SIZE({self.function_safe_name()},{i})"
                 else:       size = "0"
-                lines.append(f"{self.function_safe_name()}_DIM_{i} = {size}")
+                lines.append(f"{self.name}_DIM_{i} = {size}")
             # Copy the allocatable address only when it is present.
             if present:
                 first_pos = ",".join(["1"]*len(self.dimension))
-                lines.append(f"{self.function_safe_name()} = LOC({self.name}_LOCAL({first_pos}))")
+                lines.append(f"{self.fort_input()[-1]} = LOC({self.function_safe_name()}({first_pos}))")
         # Return the list of lines.
         return lines
 
@@ -441,8 +444,9 @@ class Argument:
         temp.name += "_LOCAL"
         # Get allocatable present input.
         if (self.allocatable):
+            decs.insert(0,"USE ISO_C_BINDING, ONLY: C_BOOL")
             args.append(f'{self.name}_ALLOCATED')
-            decs.append(f'LOGICAL, INTENT(OUT) :: {self.name}_ALLOCATED')
+            decs.append(f'LOGICAL(KIND=C_BOOL), INTENT(OUT) :: {self.name}_ALLOCATED')
             lines += [f'{self.name}_ALLOCATED = ALLOCATED({self.name})',
                       f'IF (.NOT. {self.name}_ALLOCATED) RETURN']
 
@@ -718,10 +722,18 @@ class Argument:
             elif (line[0] == "SAVE"):
                 line.pop(0)
                 self.save = True
-            # Read PARAMETER (discard this)
+            # Read PARAMETER
             elif (line[0] == "PARAMETER"):
                 line.pop(0)
                 self.parameter = True
+            # Read TARGET
+            elif (line[0] == "TARGET"):
+                line.pop(0)
+                self.target = True
+            # Read POINTER
+            elif (line[0] == "POINTER"):
+                line.pop(0)
+                self.pointer = True
             # Read EXTERNAL (discard this)
             elif (line[0] == "EXTERNAL"): 
                 line.pop(0)
